precision mediump float;
varying vec2 v_map_texture_position;
varying vec2 v_decal_texture_position;
varying float v_back_front;

uniform sampler2D map_texture;
uniform sampler2D decal_texture;

#ifdef DEBUG_RENDERING
uniform vec3 debug_color;
#endif

const lowp float threshold = 0.25;

void main() {
  lowp float shading = 1.0;
  lowp float pos;

  lowp float A = texture2D(map_texture, v_map_texture_position + vec2(-0.005, +0.005)).a;
  lowp float B = texture2D(map_texture, v_map_texture_position + vec2(-0.0015, +0.0015)).a;
  lowp float C = texture2D(map_texture, v_map_texture_position).a;
  lowp float D = texture2D(map_texture, v_map_texture_position + vec2(+0.0015, -0.0015)).a;

  if(C < threshold) {
    if(A > threshold) shading = 0.5;

    pos = 1.0;
  } else {
    if(B < threshold)      shading = 1.3;
    else if(D < threshold) shading = 0.7;

    pos = 0.0;
  }

  lowp vec4 color = texture2D(decal_texture, v_decal_texture_position);
  float back_front_alpha = 1.0 - (clamp(abs(v_back_front - pos), 0.5, 1.0) * 2.0 - 1.0);

  #ifdef DEBUG_RENDERING
  if (debug_color != vec3(0.0, 0.0, 0.0))
    color.rgb = color.rgb * 0.3 + debug_color * 0.7;
  #endif

  gl_FragColor = vec4(color.rgb * shading, color.a) * back_front_alpha;
}
